==> v0.0.1.tar.gz: ipc/cmd/ipcd/main.go <==
/*
This Source Code Form is subject to the terms of the Mozilla Public
License, v. 2.0. If a copy of the MPL was not distributed with this
file, You can obtain one at https://mozilla.org/MPL/2.0/.
*/

package main

import (
	"context"
	"errors"
	"io"
	"net"
	"os"
	"os/signal"
	"syscall"
	"time"

	"idio.link/go/ipc/common"
	"idio.link/go/ipc/protocols"
)

func handleIPCConnection(ctx context.Context, c net.Conn) {
	log := ctx.Value("log").(*common.Logger)
	log.Info("accepted connection; awaiting requests...")
	log = log.Push("handle ipc connection")

	defer func() {
		err := c.Close()
		if err != nil {
			log.Err("handle ipc connection: failed to close connection: %v", err)
		}
	}()

	cpm := protocols.NewClientPM()
	buf := make([]byte, 1024)

	for ctx.Err() == nil {
		// TODO move config into server struct
		err := c.SetDeadline(time.Now().Add(100 * time.Millisecond))
		if err != nil {
			log.Err("set connection deadline failed: %v", err)
			continue
		}
		sdu, err := common.Receive(c, buf)
		if errors.Is(err, os.ErrDeadlineExceeded) {
			continue
		}
		if errors.Is(err, io.EOF) {
			return
		}
		if err != nil {
			log.Err("%v", err)
			continue // TODO should this be a return?
		}
		sdu, err = cpm.Event(sdu)
		if err := common.Send(c, sdu); err != nil {
			if !errors.Is(err, os.ErrDeadlineExceeded) {
				log.Err("%v", err)
			}
			continue
		}
	}
}

func acceptIPCConnections(
	ctx context.Context,
	l *net.UnixListener,
) {
	log := ctx.Value("log").(*common.Logger)
	log.Info("accepting connections...") // TODO should info bypass the base msg?
	log.Push("accept ipc connections")

	defer func() {
		err := l.Close()
		if err != nil {
			log.Err("failed to close listener: %v", err)
		}
	}()

	for ctx.Err() == nil {
		// TODO move config into server struct
		err :=
			l.SetDeadline(time.Now().Add(100 * time.Millisecond))

		if err != nil {
			log.Err("set accept deadline failed: %v", err)
			continue
		}
		c, err := l.Accept()
		if err != nil {
			if !errors.Is(err, os.ErrDeadlineExceeded) {
				log.Err("listener accept failed: %+v", err)
			}
			continue
		}
		go handleIPCConnection(ctx, c)
	}
	return
}

func main() {
	log := common.NewLogger()
	ctx := context.WithValue(context.Background(), "log", log)
	ctx, cancel := context.WithCancel(ctx)

	log.Info("establish signal traps")
	go func() {
		c := make(chan os.Signal, 1)
		defer close(c)

		signal.Notify(c, syscall.SIGINT, syscall.SIGTERM)
		<-c
		cancel()
	}()

	log.Info("listen on abstract unix socket idio.link/go/ipc")

	// see `man 7 unix` for the unix sock abstract namespace
	abstract := "\000idio.link/go/ipc"
	addr, err := net.ResolveUnixAddr("unixpacket", abstract)
	if err != nil {
		panic(err)
	}
	listener, err := net.ListenUnix("unixpacket", addr)
	if err != nil {
		panic(err)
	}
	// TODO move to goroutine and supervise with restart
	acceptIPCConnections(ctx, listener)

	log.Info("exiting: %v", err)
	os.Exit(1)
}

==> v0.0.2.tar.gz: ipc/cmd/ipcd/main.go <==
/*
This Source Code Form is subject to the terms of the Mozilla Public
License, v. 2.0. If a copy of the MPL was not distributed with this
file, You can obtain one at https://mozilla.org/MPL/2.0/.
*/

package main

import (
	"context"
	"errors"
	"io"
	"net"
	"os"
	"os/signal"
	"syscall"
	"time"

	"idio.link/go/ipc/common"
	"idio.link/go/ipc/protocols"
)

func handleIPCConnection(ctx context.Context, c net.Conn) {
	log := ctx.Value("log").(*common.Logger)
	log.Print("accepted connection; awaiting requests...")
	log = log.Push("handle ipc connection")

	defer func() {
		err := c.Close()
		if err != nil {
			log.Error("handle ipc connection: failed to close connection: %v", err)
		}
	}()

	cpm := protocols.NewClientPM()
	buf := make([]byte, 1024)

	for ctx.Err() == nil {
		// TODO move config into server struct
		err := c.SetDeadline(time.Now().Add(100 * time.Millisecond))
		if err != nil {
			log.Error("set connection deadline failed: %v", err)
			continue
		}
		sdu, err := common.Receive(c, buf)
		if errors.Is(err, os.ErrDeadlineExceeded) {
			continue
		}
		if errors.Is(err, io.EOF) {
			return
		}
		if err != nil {
			log.Error("%v", err)
			continue // TODO should this be a return?
		}
		sdu, err = cpm.Event(sdu)
		if err := common.Send(c, sdu); err != nil {
			if !errors.Is(err, os.ErrDeadlineExceeded) {
				log.Error("%v", err)
			}
			continue
		}
	}
}

func acceptIPCConnections(
	ctx context.Context,
	l *net.UnixListener,
) {
	log := ctx.Value("log").(*common.Logger)
	log.Print("accepting connections...") // TODO should info bypass the base msg?
	log.Push("accept ipc connections")

	defer func() {
		err := l.Close()
		if err != nil {
			log.Error("failed to close listener: %v", err)
		}
	}()

	for ctx.Err() == nil {
		// TODO move config into server struct
		err :=
			l.SetDeadline(time.Now().Add(100 * time.Millisecond))

		if err != nil {
			log.Error("set accept deadline failed: %v", err)
			continue
		}
		c, err := l.Accept()
		if err != nil {
			if !errors.Is(err, os.ErrDeadlineExceeded) {
				log.Error("listener accept failed: %+v", err)
			}
			continue
		}
		go handleIPCConnection(ctx, c)
	}
	return
}

func main() {
	log := common.NewLogger()
	ctx := context.WithValue(context.Background(), "log", log)
	ctx, cancel := context.WithCancel(ctx)

	log.Print("establish signal traps")
	go func() {
		c := make(chan os.Signal, 1)
		defer close(c)

		signal.Notify(c, syscall.SIGINT, syscall.SIGTERM)
		s := <-c
		log.Print("received %v signal; shutting down", s)
		cancel()
	}()

	log.Print("listen on abstract unix socket idio.link/go/ipc")

	// see `man 7 unix` for the unix sock abstract namespace
	abstract := "\000idio.link/go/ipc"
	addr, err := net.ResolveUnixAddr("unixpacket", abstract)
	if err != nil {
		panic(err)
	}
	listener, err := net.ListenUnix("unixpacket", addr)
	if err != nil {
		panic(err)
	}
	// TODO move to goroutine and supervise with restart
	acceptIPCConnections(ctx, listener)

	log.Print("exiting")
}
